/****************************************************************************
* auplay.c
*
* B. Mayton <bmayton@cs.washington.edu>
* University of Washington CSE
* November 2007
*
* This program reads an audio file in .au format, prints out information
* about the file, and plays it through the OSS interface on the iMote2/
* SuperBird.
****************************************************************************/

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <sys/soundcard.h>
#include <fcntl.h>

#define AUDIO_FILE_MAGIC (uint32_t)0x02E736E64

#define AUDIO_FILE_ENCODING_MULAW 1
#define AUDIO_FILE_ENCODING_LINEAR_8 2
#define AUDIO_FILE_ENCODING_LINEAR_16 3

#define STEREO 2
#define MONO 1

// This struct defines the header format of an .au file
typedef struct {
	uint32_t magic;
	uint32_t hdr_size;
	uint32_t data_size;
	uint32_t encoding;
	uint32_t sample_rate;
	uint32_t channels;
} au_header;

int main(int argc, char **argv) {
	FILE *au_fd;
	int dsp_fd;
	au_header head;
	int encoding;
	int sample_rate;
	int channels;
	int32_t buffer[4096];
	int count;
	
	// Check arguments./	
	if(argc != 2) {
		fprintf(stderr, "Usage: %s <filename.au>\n", argv[0]);
		exit(1);
	}
	
	// Open file, exit if not successful
	au_fd = fopen(argv[1], "r");
	if(au_fd == NULL) {
		fprintf(stderr, "Error: Cannot open file %s\n", argv[1]);
		exit(1);
	}
	
	// Read header and check parameters, converting from network byte order to
	// host byte order
	fread(&head, sizeof(au_header), 1, au_fd);
	head.encoding = ntohl(head.encoding);
	head.sample_rate = ntohl(head.sample_rate);
	head.channels = ntohl(head.channels);
	head.magic = ntohl(head.magic);
	
	if( head.magic != AUDIO_FILE_MAGIC ) {
		fprintf(stderr, "Error: not an .au file\n");
		exit(1);
	}
	
	// Print out information about the audio file
	switch(head.encoding) {
		case AUDIO_FILE_ENCODING_MULAW:
			printf("uLaw 8-bit, ");
			encoding = AFMT_MU_LAW;
			break;
		case AUDIO_FILE_ENCODING_LINEAR_8:
			printf("linear 8-bit PCM, ");
			encoding = AFMT_S8;
			break;
		case AUDIO_FILE_ENCODING_LINEAR_16:
			printf("linear 16-bit PCM, ");
			encoding = AFMT_S16_BE;
			break;
		default:
			printf("unsupported encoding\n");
			exit(1);
	}
	
	sample_rate = head.sample_rate;
	printf("%d Hz, ", sample_rate);
	
	channels = head.channels;
	switch(head.channels) {
		case STEREO:
			printf("stereo\n");
			break;
		case MONO:
			printf("mono\n");
			break;
		default:
			printf("unsupported channels\n");
			exit(1);
	}
	
	// Set up the audio device
	dsp_fd = open("/dev/dsp", O_WRONLY);
	if(dsp_fd < 0) {
		fprintf(stderr, "Error: can't open /dev/dsp\n");
		exit(dsp_fd);
	}
	
	// Set parameters for the audio device to match the data in the file
	ioctl(dsp_fd, SNDCTL_DSP_SETFMT, &encoding);
	ioctl(dsp_fd, SNDCTL_DSP_SPEED, &sample_rate);
	ioctl(dsp_fd, SNDCTL_DSP_CHANNELS, &channels);
	
	printf("Opened audio device.  Encoding: %d, %d Hz, %d channels\n", encoding, sample_rate, channels);
	
	// Read the input file 4096 samples at a time, and output to the audio 
	// device
	while( !feof(au_fd) ) {
		count = fread(buffer, 4, 4096, au_fd);
		//printf("Read %d samples\n", count);
		
		if(count < 4096) {
			memset(buffer+count, 0, 4096-count);
		}
		
		write(dsp_fd, buffer, 4*4096);
		

	}
	
	printf("Finished playing the file. Closing audio device.\n");
	
	close(dsp_fd);
	fclose(au_fd);
	
	return 0;

}
