/****************************************************************************
* simplesynth.c
*
* B. Mayton <bmayton@cs.washington.edu>
* University of Washington CSE
* February 2008
*
* This library provides a simple sine-table synthesis thread, and allows
* control of the frequency of the output.
****************************************************************************/

#include <pthread.h>
#include <stdio.h>
#include <sys/soundcard.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdint.h>
#include <sys/signal.h>
#include "sinetable_8.h"

#include "simplesynth.h"

#define SIN_TABLE_READ(x) synth_wavetable[x>>(16-WAVE_TABLE_BITS)]
#define BUFFER_SIZE 512

int dsp_fd;
int errno;
int increment;

pthread_t synth_thread = 0;

/**
 * Set up and open the audio device
 */
static void synth_setup_audio_out(int encoding, int sample_rate, int channels) {
	if( (dsp_fd = open("/dev/dsp", O_RDWR)) < 0 ) {
		perror("opening /dev/dsp");
		return;
	} 
	
	ioctl(dsp_fd, SNDCTL_DSP_SETFMT, &encoding);
	ioctl(dsp_fd, SNDCTL_DSP_SPEED, &sample_rate);
	ioctl(dsp_fd, SNDCTL_DSP_CHANNELS, &channels);
	
	printf("Opened audio device. Encoding: %d, %d Hz, %d channel(s)\n", encoding, sample_rate, channels);
}

/**
 * Main function for the synthesis thread; the audio synthesis happens here
 */
void *synth_thread_runner(void *arg) {
	int i;
	int buffer_pos;
	uint16_t table_pos = 0;
	int16_t out_buffer[BUFFER_SIZE];
	
	for(;;) {
		for(buffer_pos = 0; buffer_pos < BUFFER_SIZE; buffer_pos++) {
			out_buffer[buffer_pos] = SIN_TABLE_READ(table_pos);
			table_pos += increment;
		}
		write(dsp_fd, out_buffer, BUFFER_SIZE*sizeof(int16_t));
	}
}

/**
 * Start the synthesizer
 */
void synth_start() {
	if(synth_thread == 0) {
		synth_setup_audio_out(AFMT_S16_LE, 44100, 1);
		pthread_create(&synth_thread, NULL, synth_thread_runner, NULL);
	}
}

/**
 * Stop the synthesizer
 */
void synth_stop() {
	if(synth_thread != 0) {
		pthread_kill(synth_thread, SIGINT);
		close(dsp_fd);
		synth_thread = 0;
	}
}

/**
 * Allows the frequency of the synthesizer to be changed.  There will be some
 * delay before the change is requested and the audio chip starts playing the
 * new frequency because the chip buffers some samples.
 */
void synth_set_frequency(int freq) {
	increment = 256 * freq * 256 / 44100;
}


